@@ -1,4 +1,4 @@
-# Copyright (C)  2012-2023  Mark Seligman
+# Copyright (C)  2012-2024  Mark Seligman
 ##
 ## This file is part of RboristBase.
 ##
@@ -19,5 +19,9 @@ expandfe <- function(arbOut) UseMethod("expandfe")
 
 
 expandfe.default <- function(arbOut) {
-  return (tryCatch(.Call("expandR", arbOut), error = function(e) {stop(e)}))
+  if (!inherits(arbOut, "arbTrain")) {
+    stop("Expecting an arbTrain object.")
+  }
+
+  return (tryCatch(.Call("expandTrainRcpp", arbOut), error = function(e) {stop(e)}))
 }
