@@ -106,7 +106,7 @@ struct Cand {
      few mantissa bits.  Hence using the low-order bits to arbitrate other
      choices is unlikely to introduce spurious correlations.
    */
-  inline static unsigned int getRandLow(double rVal) {
+  static unsigned int getRandLow(double rVal) {
     union { double d; uint32_t ui[2]; } u = {rVal};
     
     return u.ui[0];
