@@ -22,10 +22,6 @@
 
 using namespace std;
 
-#include "forestbridge.h"
-#include "leafbridge.h"
-#include "samplerbridge.h"
-
 
 /**
    @brief Consolidates common components required by all prediction entries.
@@ -36,75 +32,77 @@ struct PredictBridge {
   /**
      @brief Constructor boxes training and output summaries.
 
-     @param nThread is the number of OMP threads requested.
-
      Remaining parameters mirror similarly-named members.
    */
-  PredictBridge(unique_ptr<struct RLEFrame> rleFrame_,
-                ForestBridge forest_,
-		unsigned int nPermute,
-		unsigned int nThread);
-
-
-  PredictBridge(PredictBridge&&);
-
+  PredictBridge();
 
+  
   virtual ~PredictBridge();
 
-  
-  size_t getNRow() const;
 
-
-  unsigned int getNTree() const;
+  static void initPredict(bool indexing,
+			  bool bagging,
+			  unsigned int nPermute,
+			  bool trapUnobserved);
   
 
-  bool permutes() const;
+  /**
+     @brief Initializes quantile reporting.
+   */
+  static void initQuant(vector<double> quantile);
+
+
+  /**
+     @brief Initializes categorical probability recording.
+   */
+  static void initCtgProb(bool doProb);
+
+
+  static void initOmp(unsigned int nThread);
 
   
+  size_t getNObs() const;
+
+
   /**
      @brief Computes Meinshausen-style weight vectors over a set of observations.
      
      @return vector of normalized weight vectors.
    */
-  static vector<double> forestWeight(const ForestBridge& forestBridge,
-					     const SamplerBridge& samplerBridge,
-					     const LeafBridge& leafBridge,
-					     const double indices[],
-					     size_t nObs,
-					     unsigned int nThread);
-
-  
-protected:
-  unique_ptr<struct RLEFrame> rleFrame; ///< Local ownership
-  ForestBridge forestBridge; ///< Local ownership.
-  const unsigned int nPermute; ///< # times to permute.
+  static vector<double> forestWeight(const struct ForestBridge& forestBridge,
+				     const struct SamplerBridge& samplerBridge,
+				     const double indices[],
+				     size_t nObs);
 };
 
 
 struct PredictRegBridge : public PredictBridge {
-  PredictRegBridge(unique_ptr<struct RLEFrame> rleFrame_,
-		   ForestBridge forestBridge_,
-		   SamplerBridge samplerBridge_,
-		   LeafBridge leafBridge_,
-		   vector<double> yTest,
-		   unsigned int nPermute_,
-		   bool indexing,
-		   bool trapUnobserved,
-		   unsigned int nThread,
-		   vector<double> quantile_);
+  PredictRegBridge(unique_ptr<struct SummaryReg> summary_);
 
 
   ~PredictRegBridge();
 
 
+  unique_ptr<struct SummaryReg> summary;
+
+  
+
   /**
      @brief External entry for prediction.
 
      May be parametrized for separate entry in distributed setting.
    */
-  void predict() const;
+  static unique_ptr<PredictRegBridge> predict(const class Sampler* sampler,
+					      class Forest* forest,
+				       vector<double> yTest);
+
+  bool permutes() const;
 
 
+  size_t getNObs() const;
+
+
+  
   /**
      @return reference to cached index vector.
    */
@@ -117,47 +115,43 @@ struct PredictRegBridge : public PredictBridge {
   double getSSE() const;
 
 
-  const vector<double>& getSSEPermuted() const;
+  vector<vector<double>> getSSEPermuted() const;
 
   
-  const vector<double>& getYTest() const;
-  
+  vector<vector<double>> getSAEPermuted() const;
 
+  
   const vector<double>& getYPred() const;
   
   
   /**
      @return vector of predection quantiles iff quant non-null else empty.
    */
-  const vector<double> getQPred() const;
+  const vector<double>& getQPred() const;
 
   /**
      @return vector of estimate quantiles iff quant non-null else empty.
    */
-  const vector<double> getQEst() const;
-
-private:
-  SamplerBridge samplerBridge; ///< Local ownership.
-  LeafBridge leafBridge; ///< " "
-  unique_ptr<class PredictReg> predictRegCore;
+  const vector<double>& getQEst() const;
 };
 
 
 struct PredictCtgBridge : public PredictBridge {
-  PredictCtgBridge(unique_ptr<struct RLEFrame> rleFrame_,
-		   ForestBridge forestBridge_,
-		   SamplerBridge samplerBridge_,
-		   LeafBridge leafBridge_,
-		   vector<unsigned int> yTest,
-		   unsigned int nPermute_,
-		   bool doProb,
-		   bool indexing,
-		   bool trapUnobserved,
-		   unsigned int nThread);
-  
+  PredictCtgBridge(unique_ptr<struct SummaryCtg> summary);
+
+
   ~PredictCtgBridge();
 
 
+  unique_ptr<struct SummaryCtg> summary;
+  
+
+  bool permutes() const;
+
+
+  size_t getNObs() const;
+
+  
   /**
      @return reference to cached index vector.
    */
@@ -173,13 +167,13 @@ struct PredictCtgBridge : public PredictBridge {
   const vector<double>& getMisprediction() const;
 
 
-  const vector<vector<double>>& getMispredPermuted() const;
+  vector<vector<vector<double>>> getMispredPermuted() const;
   
 
   double getOOBError() const;
 
 
-  const vector<double>& getOOBErrorPermuted() const;
+  vector<vector<double>> getOOBErrorPermuted() const;
   
 
   /**
@@ -187,22 +181,19 @@ struct PredictCtgBridge : public PredictBridge {
 
      May be parametrized for separate entry in distributed setting.
    */
-  void predict() const;
+  static unique_ptr<PredictCtgBridge> predict(const class Sampler* sampler,
+					      class Forest* forest,
+					      vector<unsigned int> yTest);
+
 
   unsigned int ctgIdx(unsigned int ctgTest,
                       unsigned int ctgPred) const;
   
 
-  const unsigned int* getCensus() const;
+  const vector<unsigned int>& getCensus() const;
   
 
   const vector<double>& getProb() const;
-  
-
-private:
-  SamplerBridge samplerBridge; // Local ownership.
-  LeafBridge leafBridge; // " "
-  unique_ptr<class PredictCtg> predictCtgCore;
 };
 
 
