@@ -1,4 +1,4 @@
-// Copyright (C)  2012-2023   Mark Seligman
+// Copyright (C)  2012-2024   Mark Seligman
 //
 // This file is part of RboristBase.
 //
@@ -27,48 +27,52 @@
 #include "resizeR.h"
 #include "samplerR.h"
 #include "samplerbridge.h"
+#include "rleframeR.h"
 
 #include <algorithm>
 
 
 const string SamplerR::strYTrain = "yTrain";
 const string SamplerR::strNSamp = "nSamp";
-const string SamplerR::strNTree = "nTree";
+const string SamplerR::strNTree = "nTree"; // EXIT
+const string SamplerR::strNRep = "nRep";
 const string SamplerR::strSamples = "samples";
 const string SamplerR::strHash = "hash";
 
-
+// [[Rcpp::export]]
 RcppExport SEXP rootSample(const SEXP sY,
-			   const SEXP sRowWeight,
+			   const SEXP sWeight,
 			   const SEXP sNSamp,
 			   const SEXP sNTree,
-			   const SEXP sWithRepl) {
-  BEGIN_RCPP
-
-  NumericVector weight;
-  if (!Rf_isNull(sRowWeight)) {
-    NumericVector rowWeight(as<NumericVector>(sRowWeight));
-    weight = rowWeight / sum(rowWeight);
+			   const SEXP sWithRepl,
+			   const SEXP sNHoldout,
+			   const SEXP sIdxUndefined) {
+  NumericVector weight(as<NumericVector>(sWeight));
+  vector<size_t> undefined;
+  if (Rf_isInteger(sIdxUndefined)) { // Index type specified by front end.
+    IntegerVector undefinedFE(as<NumericVector>(sIdxUndefined));
+    undefined = vector<size_t>(undefinedFE.begin(), undefinedFE.end());
+  }
+  else {
+    NumericVector undefinedFE(as<NumericVector>(sIdxUndefined));
+    undefined = vector<size_t>(undefinedFE.begin(), undefinedFE.end());
   }
 
-  return SamplerR::rootSample(sY, sNSamp, sNTree, sWithRepl, weight);
-
-  END_RCPP
+  return SamplerR::rootSample(sY, sNSamp, sNTree, sWithRepl, vector<double>(weight.begin(), weight.end()), sNHoldout, undefined);
 }
 
 
+// [[Rcpp::export]]
 List SamplerR::rootSample(const SEXP sY,
 			  const SEXP sNSamp,
 			  const SEXP sNTree,
 			  const SEXP sWithRepl,
-			  const NumericVector& weight) {
-  BEGIN_RCPP
-
-  SamplerBridge bridge(as<size_t>(sNSamp), getNObs(sY), as<unsigned int>(sNTree), as<bool>(sWithRepl), weight.length() == 0 ? nullptr : &weight[0]);
-  sampleTrees(bridge);
+			  const vector<double>& weight,
+			  const SEXP sNHoldout,
+			  const vector<size_t>& undefined) {
+  SamplerBridge bridge(as<size_t>(sNSamp), getNObs(sY), as<unsigned int>(sNTree), as<bool>(sWithRepl), weight, as<size_t>(sNHoldout), undefined);
+  sampleRepeatedly(bridge);
   return wrap(bridge, sY);
-
-  END_RCPP
 }
 
 
@@ -83,9 +87,9 @@ size_t SamplerR::getNObs(const SEXP& sY) {
 }
 
 
-void SamplerR::sampleTrees(SamplerBridge& bridge) {
+void SamplerR::sampleRepeatedly(SamplerBridge& bridge) {
   // May be parallelized if a thread-safe PRNG is available.
-  for (unsigned int tIdx = 0; tIdx < bridge.getNTree(); tIdx++) {
+  for (unsigned int tIdx = 0; tIdx < bridge.getNRep(); tIdx++) {
     bridge.sample();
   }
 }
@@ -99,30 +103,25 @@ vector<size_t> SamplerR::sampleObs(size_t nSamp,
 }
 
 
+// [[Rcpp::export]]
 IntegerVector SamplerR::sampleReplace(NumericVector& weight,
 				     size_t nSamp) {
-  BEGIN_RCPP
   RNGScope scope;
-  IntegerVector rowSample(sample(weight.length(), nSamp, true, weight, false));
-  return rowSample;
-  END_RCPP
+  return IntegerVector(sample(weight.length(), nSamp, true, weight, false));
 }
 
 
- IntegerVector SamplerR::sampleNoReplace(NumericVector& weight,
+// [[Rcpp::export]]
+IntegerVector SamplerR::sampleNoReplace(NumericVector& weight,
 					 size_t nSamp) {
-  BEGIN_RCPP
   RNGScope scope;
-  IntegerVector rowSample(sample(weight.length(), nSamp, false, weight, false));
-  return rowSample;
-  END_RCPP
+  return IntegerVector(sample(weight.length(), nSamp, false, weight, false));
 }
 
 
+// [[Rcpp::export]]
 List SamplerR::wrap(const SamplerBridge& bridge,
 		    const SEXP& sY) {
-  BEGIN_RCPP
-
   List sampler;
   // Caches the front end's response vector as is.
   if (Rf_isFactor(sY)) {
@@ -138,24 +137,21 @@ List SamplerR::wrap(const SamplerBridge& bridge,
   sampler.attr("class") = "Sampler";
 
   return sampler;
-
-  END_RCPP
 }
 
 
+// [[Rcpp::export]]
 List SamplerR::wrap(const SamplerBridge& bridge,
 		    const IntegerVector& yTrain) {
-  BEGIN_RCPP
-
   List sampler = List::create(_[strYTrain] = yTrain,
-			      _[strSamples] = std::move(bridgeConsume(bridge)),
-			      _[strNSamp] = bridge.getNSamp(),
-			      _[strNTree] = bridge.getNTree(),
-			      _[strHash] = 0
-			);
-
+		      _[strSamples] = std::move(bridgeConsume(bridge)),
+		      _[strNSamp] = bridge.getNSamp(),
+		      _[strNRep] = bridge.getNRep(),
+		      _[strNTree] = bridge.getNRep(),
+		      _[strHash] = 0
+		      );
+  sampler.attr("class") = "Sampler";
   return sampler;
-  END_RCPP
 }
 
 
@@ -166,27 +162,24 @@ NumericVector SamplerR::bridgeConsume(const SamplerBridge& bridge) {
 }
 
 
+// [[Rcpp::export]]
 List SamplerR::wrap(const SamplerBridge& bridge,
 		    const NumericVector& yTrain) {
-  BEGIN_RCPP
-
   List sampler = List::create(_[strYTrain] = yTrain,
 			      _[strSamples] = std::move(bridgeConsume(bridge)),
 			      _[strNSamp] = bridge.getNSamp(),
-			      _[strNTree] = bridge.getNTree(),
+			      _[strNRep] = bridge.getNRep(),
+			      _[strNTree] = bridge.getNRep(),
 			      _[strHash] = 0
 			);
   sampler.attr("class") = "Sampler";
   return sampler;
-
-  END_RCPP
 }
 
 
-SamplerBridge SamplerR::unwrapTrain(const List& lSampler,
-				    const List& argList) {
+SamplerBridge SamplerR::unwrapTrain(const List& lSampler) {
   if (Rf_isFactor((SEXP) lSampler[strYTrain])) {
-    return makeBridgeTrain(lSampler, as<IntegerVector>(lSampler[strYTrain]), argList);
+    return makeBridgeTrain(lSampler, as<IntegerVector>(lSampler[strYTrain]));
   }
   else {
     return makeBridgeTrain(lSampler, as<NumericVector>(lSampler[strYTrain]));
@@ -195,20 +188,18 @@ SamplerBridge SamplerR::unwrapTrain(const List& lSampler,
 
 
 SamplerBridge SamplerR::makeBridgeTrain(const List& lSampler,
-					const IntegerVector& yTrain,
-					const List& argList) {
-  return SamplerBridge(std::move(coreCtg(yTrain)),
+					const IntegerVector& yTrain) {
+  return SamplerBridge(coreCtg(yTrain),
 		       as<size_t>(lSampler[strNSamp]),
 		       as<unsigned int>(lSampler[strNTree]),
 		       Rf_isNull(lSampler[strSamples]) ? nullptr : NumericVector((SEXP) lSampler[strSamples]).begin(),
-		       as<CharacterVector>(yTrain.attr("levels")).length(),
-		       std::move(ctgWeight(yTrain, as<NumericVector>(argList["classWeight"]))));
+		       as<CharacterVector>(yTrain.attr("levels")).length());
 }
 
 
 SamplerBridge SamplerR::makeBridgeTrain(const List& lSampler,
 					const NumericVector& yTrain) {
-  return SamplerBridge(std::move(vector<double>(yTrain.begin(), yTrain.end())),
+  return SamplerBridge(vector<double>(yTrain.begin(), yTrain.end()),
 		       as<size_t>(lSampler[strNSamp]),
 		       as<unsigned int>(lSampler[strNTree]),
 		       Rf_isNull(lSampler[strSamples]) ? nullptr : NumericVector((SEXP) lSampler[strSamples]).begin());
@@ -217,44 +208,21 @@ SamplerBridge SamplerR::makeBridgeTrain(const List& lSampler,
 
 vector<unsigned int> SamplerR::coreCtg(const IntegerVector& yTrain) {
   IntegerVector yZero = yTrain - 1;
-  vector<unsigned int> yTrainCore(yZero.begin(), yZero.end());
-  return yTrainCore;
-}
-
-
-vector<double> SamplerR::ctgWeight(const IntegerVector& yTrain,
-				   const NumericVector& classWeight) {
-  IntegerVector yZero = yTrain - 1;
-  auto scaledWeight = clone(classWeight);
-  // Default class weight is all unit:  scaling yields 1.0 / nCtg uniformly.
-  // All zeroes is a place-holder to indicate balanced scaling:  class weights
-  // are proportional to the inverse of the count of the class in the response.
-  if (is_true(all(classWeight == 0.0))) {
-    NumericVector tb(table(yZero));
-    for (R_len_t i = 0; i < classWeight.length(); i++) {
-      scaledWeight[i] = tb[i] == 0.0 ? 0.0 : 1.0 / tb[i];
-    }
-  }
-  NumericVector yWeighted = scaledWeight / sum(scaledWeight); // in [0,1]
-  NumericVector weight = yWeighted[yZero];
-
-  vector<double> weightVec(weight.begin(), weight.end());
-  return weightVec;
+  return vector<unsigned int>(yZero.begin(), yZero.end());
 }
 
 
 SamplerBridge SamplerR::unwrapPredict(const List& lSampler,
 				      const List& lDeframe,
-				      const List& lArgs) {
-  bool bagging = as<bool>(lArgs["bagging"]);
+				      bool bagging) {
   if (bagging)
     checkOOB(lSampler, lDeframe);
 
   if (Rf_isNumeric((SEXP) lSampler[strYTrain])) {
-    return makeBridgeNum(lSampler, bagging);
+    return makeBridgeNum(lSampler, lDeframe);
   }
   else if (Rf_isFactor((SEXP) lSampler[strYTrain])) {
-    return makeBridgeCtg(lSampler, bagging);
+    return makeBridgeCtg(lSampler, lDeframe);
   }
   else {
     stop("Unrecognized training response type");
@@ -262,50 +230,55 @@ SamplerBridge SamplerR::unwrapPredict(const List& lSampler,
 }
 
 
-SEXP SamplerR::checkOOB(const List& lSampler, const List& lDeframe) {
-  BEGIN_RCPP
+// [[Rcpp::export]]
+void SamplerR::checkOOB(const List& lSampler, const List& lDeframe) {
   if (Rf_isNull(lSampler[strSamples]))
     stop("Out-of-bag prediction requested with empty sampler.");
 
   if (getNObs(lSampler[strYTrain]) != as<size_t>((SEXP) lDeframe["nRow"]))
     stop("Bag and prediction row counts do not agree.");
-
-  END_RCPP
 }
 
 
 SamplerBridge SamplerR::makeBridgeNum(const List& lSampler,
-				      bool bagging) {
+				      const List& lDeframe,
+				      bool generic) {
   NumericVector yTrain(as<NumericVector>(lSampler[strYTrain]));
-  return SamplerBridge(std::move(vector<double>(yTrain.begin(), yTrain.end())),
+  return SamplerBridge(vector<double>(yTrain.begin(), yTrain.end()),
 		       as<size_t>(lSampler[strNSamp]),
 		       as<unsigned int>(lSampler[strNTree]),
 		       Rf_isNull(lSampler[strSamples]) ? nullptr : NumericVector((SEXP) lSampler[strSamples]).begin(),
-		       bagging);
+		       generic ? nullptr : RLEFrameR::unwrap(lDeframe));
 }
 
 
 SamplerBridge SamplerR::makeBridgeCtg(const List& lSampler,
-				   bool bagging) {
+				      const List& lDeframe,
+				      bool generic) {
   IntegerVector yTrain(as<IntegerVector>(lSampler[strYTrain]));
-  return SamplerBridge(std::move(coreCtg(yTrain)),
+  return SamplerBridge(coreCtg(yTrain),
 		       as<CharacterVector>(yTrain.attr("levels")).length(),
 		       as<size_t>(lSampler[strNSamp]),
 		       as<unsigned int>(lSampler[strNTree]),
 		       Rf_isNull(lSampler[strSamples]) ? nullptr : NumericVector((SEXP) lSampler[strSamples]).begin(),
-		       bagging);
+		       generic ? nullptr : RLEFrameR::unwrap(lDeframe));
 }
 
 
 SamplerBridge SamplerR::unwrapGeneric(const List& lSampler) {
-  return SamplerBridge(getNObs(lSampler[strYTrain]),
-		       Rf_isNull(lSampler[strSamples]) ? nullptr : NumericVector((SEXP) lSampler[strSamples]).begin(),
-		       as<size_t>(lSampler[strNSamp]),
-		       as<unsigned int>(lSampler[strNTree]));
+  List lDummy;
+  if (Rf_isNumeric(lSampler[strYTrain]))
+    return makeBridgeNum(lSampler, lDummy, true);
+  else
+    return makeBridgeCtg(lSampler, lDummy, true);
 }
 
 
-SamplerExpand SamplerR::unwrapExpand(const List& lTrain) {
-  List lSampler(as<List>(lTrain["sampler"]));  
-  return SamplerExpand(lSampler[strNTree], getNObs(lSampler[strYTrain]));
+unsigned int SamplerR::getNRep(const List& lSampler) {
+  return as<unsigned int>(lSampler[strNTree]);
+}
+
+
+SamplerExpand SamplerExpand::unwrap(const List& lSampler) {
+  return SamplerExpand(lSampler[SamplerR::strNTree], SamplerR::getNObs(lSampler[SamplerR::strYTrain]));
 }
